function[varargout]=about_floats(varargin)
%ABOUT_FLOATS  Historical dataset of eddy-resolving subsurface floats. 
%   _______________________________________________________________________
%   
%   *|* floats.png --- Figure of the Lagragian float dataset FLOATS.MAT.  
%   With JLAB installed, type 'jhelp about_floats' to view this image. *|*
%   _______________________________________________________________________
%
%   FLOATS.MAT and FLOATS.NC contain all available historical eddy-
%   resolving subsurface float data, as distributed online at NOAA's 
%   Atlantic Oceanographic and Meteorological Laboratory (AOML)
%
%       http://www.aoml.noaa.gov/phod/float_traj/data.php 
%
%   and described by Ramsey, Furey, and Bower (2018) at
%
%       https://eos.org/project-updates/...
%            deep-floats-reveal-complex-ocean-circulation-patterns.
%
%   A list of experiments can be found at 
%
%      http://www.aoml.noaa.gov/phod/float_traj/files/...
%             FloatDatabaseTable.pdf.
%
%   FLOATS is a slightly reprocessed version of the Matlab file distributed
%   by AOML, modified to make the dataset more amenable to time series
%   analysis, which require continuous, uniformly spaced fields. 
%
%   FLOATS.MAT is a Matlab version, while FLOATS.NC is a NetCDF version.
%
%   FLOATS is distributed as a part of JDATA, a supplement to the software
%   toolbox JLAB, and is available at http://www.jmlilly.net.
%   _______________________________________________________________________
%
%   Additional processing
%
%   The following processing steps have been applied to AOML's version:
%
%       1. All data points lacking a valid position are removed
%       2. Trajectories are split at gaps exceeding two days
%       3. Trajectory segments less than two weeks in duration are removed
%       4. Data are interpolated onto a uniformly spaced time grid 
%       5. Trajectory segments with 12.5% or more missing data are removed
%       6. Velocities are computed by central differencing position
%
%   Step 4 handles the fact that not all data are uniformly spaced.  The 
%   interpolation onto a uniformly spaced time grid using the median
%   value of the sample interval for each data segment as the time spacing.
%
%   Step 5 removes 7 floats, all from the from the same experiment, which
%   have nearly half of their data missing within the uniform time grid.
%
%   Step 6 forms velocities by calling the jLab function LATLON2UV.
%   _______________________________________________________________________
%
%   Matlab version
%
%   LOAD FLOATS loads the structure FLOATS, with the following fields:
%
%       floats.about         Pointer to this document       
%       floats.experiment    Experiment names            {52 x 1 cell}
%       floats.expid         Experiment id number        [3091 x 1 array] 
%       floats.type          Float types                 {5 x 1 cell}
%       floats.typeid        Type id number              [3091 x 1 array] 
%       floats.id            Float id number             [3091 x 1 array] 
%       floats.dt            Sample interval in days     [3091 x 1 array] 
%       floats.num           Date in DATENUM format      {3091 x 1 cell} 
%       floats.lat           Latitude                    {3091 x 1 cell} 
%       floats.lon           Longitude                   {3091 x 1 cell} 
%       floats.u             Eastward velocity in cm/s   {3091 x 1 cell} 
%       floats.v             Northward velocity in cm/s  {3091 x 1 cell} 
%       floats.p             Pressure in decibar         {3091 x 1 cell} 
%       floats.t             Temperature in centigrade   {3091 x 1 cell} 
%       floats.filled        True if gap was filled      {3091 x 1 cell} 
%
%   ID is the float number, the same as 'indexFlt' in AOML's dataset.
%   EXPID is the experiment number, the same as 'expNum' in AOML's dataset.
%
%   Unlike in AOML's dataset, here floats trajectories may be split into
%   multiple segements in order to avoid having long gaps.  Thus, there are 
%   3091 trajectory segments, but only 2193 different floats. 
%
%   Missing data points in P and T are filled with INFs.  There is no 
%   missing data in the other fields. 
%
%   The field FILLED is true if a gap was filled during processing Step 4
%   listed above. Information on gaps filled during upstream processing in 
%   forming the fields that compose the AOML dataset is not available. 
%
%   The last eight field are all cell arrays, with one drifter per cell. 
%   See JCELL for a number of tools for working with this type of data. In
%   particular, CELLPLOT will plot such data.
%
%   After 'load floats, type 'use floats' to map the structure fields 
%   into named variables in the workspace--id, num, lat, lon, etc.
%
%   To convert the cell arrays into one long column vector, with a NAN 
%   separating each float, type 'cell2col(num,lat,lon,u,v,t,p);'.
%
%   The experiment names are found by experiment(expid(100)), for example,
%   which returns the experiment name of the 100th trajectory segment.
%   This will match the listing in the table FloatDatabaseTable.pdf.
%
%   The float type for a particular segment is similarly returned by e.g. 
%   type(typeid(100)).  The types are as follows: 1) SOFAR 2) Bobber SOFAR
%   3) RAFOS 4) RAFOS - VOCHA and 5) APEX.
%
%   To generate the above figure of the dataset, type 'about_floats --f'. 
%
%   FLOATS.MAT is distributed as a part of JDATA, a supplement to the 
%   software toolbox JLAB, and is available at http://www.jmlilly.net.
%   __________________________________________________________________
%
%   NetCDF version
% 
%   The NetCDF version is basically the same as the mat-file version, but
%   (i) lacking the 'experiment' and 'type' variables, which are cell 
%   arrays of strings, and (ii) having all the other cell array fields  
%   stored as column arrays with separating NaNs, see COL2CELL.
%
%   The experiment names corresponding to 'expid' can be found in the
%   document FloatDatabaseTable.pdf, while the type names are given above.
%  
%   Use the JLAB routine NCLOAD to load the NetCDF file and convert all the
%   NaN-separated columns into cells, i.e. 'ncload floats'.
%   __________________________________________________________________
%
%   Dataset creation
%
%   For completeness, the m-file ABOUT_FLOATS also contains the processing 
%   steps used in the creation of FLOATS.MAT and FLOATS.NC.
%
%   If you wish to recreate these yourself, with JLAB on your search path, 
%   'about floats --create' will create FLOATS.MAT and FLOATS.NC by 
%   reading in allFloats_12122017.mat as downloaded from AOML.  This 
%   dataset therefore needs to be on your search path. 
%   __________________________________________________________________
%
%   See also JDATA, ABOUT_DRIFTERS.
%
%   'about_floats --f' generates the sample figure shown above.
%
%   Usage: about_floats
%          about_floats --create     
%   __________________________________________________________________
%   This is part of JLAB --- type 'help jlab' for more information
%   (C) 2014--2019 J.M. Lilly --- type 'help jlab_license' for details    


if nargin==0
    help about_floats
elseif nargin>0
    if strcmpi(varargin{1}, '--create')
        floats_create,return
    elseif strcmpi(varargin{1}, '--f')
        floats_fig,return
    end
end

function[]=floats_create
        
disp('Creating FLOATS.MAT.  This may take while...')

load allFloats_12122017
num=dtnum;clear dtnum

%cv=celladd(u,cellmult(1i,v));

typeList={'SOFAR','Bobber SOFAR','RAFOS','RAFOS - VOCHA','APEX'};
indexType=nan*indexFlt;
for i=1:length(num)
    if strcmp(fltType{i},'SOFAR')
        indexType(i)=1;
    elseif strcmp(fltType{i},'Bobber SOFAR')
        indexType(i)=2;
    elseif strcmp(fltType{i},'RAFOS')
        indexType(i)=3;
    elseif strcmp(fltType{i},'RAFOS - VOCHA')
        indexType(i)=4;
    elseif strcmp(fltType{i},'APEX')
        indexType(i)=5;
    end
end
clear fltType
clear indexFlt
%--------------------------------------------------------------------------
%remove all data points with missing positions
for i=1:length(num)
   bool=isfinite(lat{i})&isfinite(lon{i});
    [num{i},lat{i},lon{i},u{i},v{i},t{i},p{i}]=...
        vindex(num{i},lat{i},lon{i},u{i},v{i},t{i},p{i},bool,1);
end
%--------------------------------------------------------------------------
%remove a repeated time point
jj=478;
num{jj}=num{jj}(1:end-1);
lat{jj}=lat{jj}(1:end-1);
lon{jj}=lon{jj}(1:end-1);
u{jj}=u{jj}(1:end-1);
v{jj}=v{jj}(1:end-1);
p{jj}=p{jj}(1:end-1);
t{jj}=t{jj}(1:end-1);
%--------------------------------------------------------------------------
%Split at gaps 
[dt,sigdt,meddt,maxdt]=sampletimes(num);
id=num;
for i=1:length(num)
    id{i}=i+0*id{i};
end

vsize(num,id,lat,lon,u,v,p,t)
cellsplit(num,id,lat,lon,u,v,p,t,2);        %Split with longest gap at 2 days
vsize(num,id,lat,lon,u,v,p,t)

[dt,sigdt,meddt,maxdt]=sampletimes(num);
%--------------------------------------------------------------------------
%remove segments less than 2 weeks in length
%vindex(num,id,lat,lon,u,v,p,t,cellength(num)>=10,1);
vindex(num,id,lat,lon,u,v,p,t,(cellmax(num)-cellmin(num))>=14,1);
vsize(num,id,lat,lon,u,v,p,t)
%--------------------------------------------------------------------------
%interpolate to a uniform time base
make original num lat lon u v p t
[dt,sigdt,meddt,maxdt]=sampletimes(num);
cellgrid(num,lat,lon,u,v,p,t,meddt);

%plot(num{1},lat{1}),hold on,plot(original.num{1},original.lat{1})

nearest=num;
parfor i=1:length(num)
    i
    nearest{i}=min(abs(vrep(num{i},size(original.num{i},1),2)...
        -vrep(original.num{i}',size(num{i},1),1)),[],2);
end
filled=num;
for i=1:length(num)
    %true if filled
    filled{i}=nearest{i}>(2/3)*meddt(i);
end
fillfrac=cellsum(filled)./cellength(filled);


%cellplot(180,lon,lat,'b'),hold on
%for i=1:length(num)
%    plot(lon{i}(filled{i}),lat{i}(filled{i}),'r+')
%end
%let's look for floats that have a large proportion of filled data
%figure,plot(fillfrac,'+')
%Wow, there are some that have 80% filled! (this was from before)
%What was happening was that cellsplit was not applicable because num
%had valid data points even when lat and lon had NaNs, before above changes
%--------------------------------------------------------------------------
%next move those with too much missing data
%figure,cellplot(yearfrac(num(fillfrac>0.1)),filled(fillfrac>0.1)),yoffset 1.

expName(cellfirst(id(fillfrac>0.125)))
%this removes 7 floats from the LAST experiment which have nearly half
%their data missing

vindex(num,id,lat,lon,u,v,p,t,filled,fillfrac,fillfrac<=0.125,1);
%--------------------------------------------------------------------------
%now find the float ids etc
id=cellfirst(id);
indexExp=indexExp(id);
indexType=indexType(id);
expName=expName(id);
expOrg=expOrg(id);
expPI=expPI(id);

[dt,sigdt,meddt,maxdt]=sampletimes(num);
length(find(sigdt==0))  %3091
length(num)%3091 segments
length(find(isfinite(cell2col(num)))) %1264149 data points
length(find(diff(sort(id))~=0))+1  %2099 floats

expid=id;
n=1;
for i=2:length(num)
    if ~strcmp(expName{i},expName{i-1})
        n=n+1;
    end
    expid(i)=n;
end
expid(end)=n;
for i=1:n
    index=find(expid==i);
    experiment{i,1}=expName{index(1)};
end

about=['For more information, type ''about_floats''.'];

%where do u and v come from?

type=typeList;
typeid=indexType;

cv=latlon2uv(num,lat,lon);
u2=cellreal(cv);
v2=cellimag(cv);

%[mat,xmid,ymid]=twodhist(cellmult(100,u),u2,[-100:100],[-100:100]);
%jpcolor(xmid,ymid,log10(mat))

%[mat,xmid,ymid]=twodhist(cellmult(100,v),v2,[-100:100],[-100:100]);
%jpcolor(xmid,ymid,log10(mat))

u=u2;
v=v2;
matsave floats about experiment expid type typeid id dt num lat lon u v p t filled
 
% cellplot(180,lon,lat,'b'),hold on
% for i=1:length(num)
%     plot(lon{i}(filled{i}),lat{i}(filled{i}),'r+')
% end

% d=num;
% for i=1:length(num)
%     d{i}=spheredist(lat{i},lon{i});
% end
% figure
% for i=1:length(num),hold on
%     plot(d{i}(filled{i}))
%     %plot(lat{i}(filled{i}),d{i}(filled{i}))
% end
% figure
% for i=1:length(num),hold on
%     plot(lat{i}(filled{i}))
%     %plot(lat{i}(filled{i}),d{i}(filled{i}))
% end
% figure
% for i=1:length(num),hold on
%     plot(num{i}(filled{i}))
%     %plot(lat{i}(filled{i}),d{i}(filled{i}))
% end


%figure,cellplot(yearfrac(num(fillfrac>0.05)),filled(fillfrac>0.05)),yoffset 1.2
% figure,cellplot(180,lon(fillfrac>0.125),lat(fillfrac>0.125))
% figure,cellplot(yearfrac(num(fillfrac>0.125)),lat(fillfrac>0.125))
% figure,cellplot(yearfrac(num(fillfrac>0.125)),filled(fillfrac>0.125)),yoffset 1.2
% figure,cellplot(yearfrac(num(fillfrac>0.05)),filled(fillfrac>0.05)),yoffset 1.2
% figure,cellplot(yearfrac(num(fillfrac>0.125)),filled(fillfrac>0.125)),yoffset 1.2
% figure,cellplot(yearfrac(original.num(fillfrac>0.25)),original.lat(fillfrac>0.25))
% figure,cellplot(180,lon(fillfrac>0.25),lat(fillfrac>0.25))
% figure,cellplot(yearfrac(num(fillfrac>0.25)),filled(fillfrac>0.25)),yoffset 1.2


%should also interpolate these to be by chronological order 
%The APEX floats have gaps in the pressure record---why?


disp('FLOATS complete.')

%Test for no internal NANs.
% clear Lnans
% for i=1:length(lat)
%     Lnans(i)=length(find(isnan(lat{i})|isnan(lon{i})));
% end
% %Good, no NaNs.

%--------------------------------------------------------------------------
%Create the NetCDF version of the dataset 
filename='floats.nc';
use floats

cd([whichdir('jlab_license')])
cd ..
cd jData
cd floats   
    
cell2col(num,lat,lon,u,v,p,t,filled);

nccreate(filename,'expid','dimensions',{'segment',length(id)}, 'Datatype','double');
nccreate(filename,'typeid','dimensions',{'segment',length(id)}, 'Datatype','double');
nccreate(filename,'id','dimensions',{'segment',length(id)}, 'Datatype','double');
nccreate(filename,'num','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'lat','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'lon','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'u','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'v','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'p','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'t','dimensions',{'column',length(num)}, 'Datatype','double');
nccreate(filename,'filled','dimensions',{'column',length(num)}, 'Datatype','double');

ncwriteatt(filename,'/','title','FLOATS.NC subsurface float dataset')
ncwriteatt(filename,'/','link','http://www.aoml.noaa.gov/phod/float_traj/data.php')
ncwriteatt(filename,'/','about','For more information, see ABOUT_FLOATS.M.')
ncwriteatt(filename,'/','note','In Matlab, try ''ncload floats'' using jLab routine ncload.')
ncwriteatt(filename,'/','creator','J. M. Lilly') 
ncwriteatt(filename,'/','timestamp',datestr(now))

ncwrite(filename,'expid',expid);
ncwrite(filename,'typeid',typeid);
ncwrite(filename,'id',id);
ncwrite(filename,'num',num);
ncwrite(filename,'lat',lat);
ncwrite(filename,'lon',lon);
ncwrite(filename,'u',u);
ncwrite(filename,'v',v);
ncwrite(filename,'p',p);
ncwrite(filename,'t',t);
ncwrite(filename,'filled',filled);
%--------------------------------------------------------------------------

function[]=floats_fig
figure
load floats,use floats
cellplot(180,lon,lat),topoplot
latratio(30)
title('Deep Lagrangian Float Trajectories from FLOATS.MAT')
%axis([-80 54 -50 78])
if 0
    currentdir=pwd;
    cd([whichdir('jlab_license') '/figures'])
    print -dpng floats
    crop floats.png
    cd(currentdir)
end

